@@ -1,44 +1,30 @@
 /*
  * Copyright (c) 1996, 1997, 1998, 1999
- *            Shigio Yamaguchi. All rights reserved.
- * Copyright (c) 1999
- *            Tama Communications Corporation. All rights reserved.
+ *             Shigio Yamaguchi. All rights reserved.
+ * Copyright (c) 1999, 2000, 2001
+ *             Tama Communications Corporation. All rights reserved.
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by Tama Communications
- *      Corporation and its contributors.
- * 4. Neither the name of the author nor the names of any co-contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
- * 
- * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
+ * This file is part of GNU GLOBAL.
  *
- *      gtagsop.c                               12-Dec-99
+ * GNU GLOBAL is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2, or (at your option)
+ * any later version.
  *
+ * GNU GLOBAL is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
 #ifndef _GTOP_H_
 #define _GTOP_H_
 #include <stdio.h>
+#include <ctype.h>
 
 #include "gparam.h"
 #include "dbop.h"
@@ -69,6 +55,11 @@
 /* gtagsfirst() */
 #define GTOP_KEY		1	/* read key part */
 #define GTOP_PREFIX		2	/* prefixed read */
+#define GTOP_NOSOURCE		4	/* don't read source file */
+#define GTOP_NOREGEX		8	/* don't use regular expression */
+#define GTOP_IGNORECASE		16	/* ignore case distinction */
+
+#define isregexchar(c)	(regexchar[c])
 
 typedef struct {
 	DBOP	*dbop;			/* descripter of DBOP */
@@ -81,13 +72,14 @@ typedef struct {
 	/*
 	 * Stuff for compact format
 	 */
-	int	opened;			/* wether or not file opened */
+	int	opened;			/* whether or not file opened */
 	char	*line;			/* current record */
 	char	tag[IDENTLEN+1];	/* current tag */
 	char	prev_tag[IDENTLEN+1];	/* previous tag */
 	char	path[MAXPATHLEN+1];	/* current path */
 	char	prev_path[MAXPATHLEN+1];/* previous path */
 	STRBUF	*sb;			/* string buffer */
+	STRBUF	*ib;			/* input buffer */
 	FILE	*fp;			/* descriptor of 'path' */
 	char	*lnop;			/* current line number */
 	int	lno;			/* integer value of 'lnop' */
@@ -97,6 +89,7 @@ const char *dbname(int);
 void	makecommand(char *, char *, STRBUF *);
 int	formatcheck(char *, int);
 int	notnamechar(char *);
+int	isregex(char *);
 GTOP	*gtagsopen(char *, char *, int, int, int);
 void	gtagsput(GTOP *, char *, char *);
 char	*gtagsget(GTOP *, char *);
